<?php
/*
 * Copyright 2005 - 2009  Zarafa B.V.
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License, version 3, 
 * as published by the Free Software Foundation with the following additional 
 * term according to sec. 7:
 *  
 * According to sec. 7 of the GNU Affero General Public License, version
 * 3, the terms of the AGPL are supplemented with the following terms:
 * 
 * "Zarafa" is a registered trademark of Zarafa B.V. The licensing of
 * the Program under the AGPL does not imply a trademark license.
 * Therefore any rights, title and interest in our trademarks remain
 * entirely with us.
 * 
 * However, if you propagate an unmodified version of the Program you are
 * allowed to use the term "Zarafa" to indicate that you distribute the
 * Program. Furthermore you may use our trademarks where it is necessary
 * to indicate the intended purpose of a product or service provided you
 * use it in accordance with honest practices in industrial or commercial
 * matters.  If you want to propagate modified versions of the Program
 * under the name "Zarafa" or "Zarafa Server", you may only do so if you
 * have a written permission by Zarafa B.V. (to acquire a permission
 * please contact Zarafa at trademark@zarafa.com).
 * 
 * The interactive user interface of the software displays an attribution
 * notice containing the term "Zarafa" and/or the logo of Zarafa.
 * Interactive user interfaces of unmodified and modified versions must
 * display Appropriate Legal Notices according to sec. 5 of the GNU
 * Affero General Public License, version 3, when you propagate
 * unmodified or modified versions of the Program. In accordance with
 * sec. 7 b) of the GNU Affero General Public License, version 3, these
 * Appropriate Legal Notices must retain the logo of Zarafa or display
 * the words "Initial Development by Zarafa" if the display of the logo
 * is not reasonably feasible for technical reasons. The use of the logo
 * of Zarafa in Legal Notices is allowed for unmodified and modified
 * versions of the software.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 *  
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */

?>
<?php
/*
 * Copyright (C) 2005 Alfresco, Inc.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.

 * As a special exception to the terms and conditions of version 2.0 of 
 * the GPL, you may redistribute this Program in connection with Free/Libre 
 * and Open Source Software ("FLOSS") applications as described in Alfresco's 
 * FLOSS exception.  You should have recieved a copy of the text describing 
 * the FLOSS exception, and it is also available here: 
 * http://www.alfresco.com/legal/licensing"
 */

require_once("Functions.php");

class ContentData extends BaseObject
{	
	private $_isPopulated = false;
	private $_isDirty = false;
	
	private $_node;
	private $_property;
	
	private $_mimetype;
	private $_size;
	private $_encoding;	
	private $_url;
	private $_newContent;
	private $_newFileContent;
	
	public function __construct($node, $property, $mimetype=null, $encoding=null, $size=-1)
	{
		$this->_node = $node;
		$this->_property = $property;
		$this->_mimetype = $mimetype;
		$this->_encoding = $encoding;
		if ($size != -1)
		{
			$this->size = $size;
		}		
		$this->_isPopulated = false;
	}	
	
	public function setPropertyDetails($node, $property)
	{
		$this->_node = $node;
		$this->_property = $property;
	}
	
	public function __toString()
	{
		$this->populateContentData();
	}
	
	public function getNode()
	{
		return $this->_node;
	}
	
	public function getProperty()
	{
		return $this->_property;
	}
	
	public function getIsDirty()
	{
		return $this->_isDirty;
	}
	
	public function getMimetype()
	{
		$this->populateContentData();
		return $this->_mimetype;
	}
	
	public function setMimetype($mimetype)
	{
		$this->populateContentData();
		$this->_mimetype = $mimetype;
	}
	
	public function getSize()
	{
		$this->populateContentData();
		return $this->_size;
	}
	
	public function getEncoding()
	{
		$this->populateContentData();
		return $this->_encoding;
	}
	
	public function setEncoding($encoding)
	{
		$this->populateContentData();
		$this->_encoding = $encoding;
	}
	
	public function getUrl()
	{
		// TODO what should be returned if the content has been updated??
		
		$this->populateContentData();
		$result = null;
		if ($this->_url != null)
		{	
			$result = $this->_url."?ticket=".$this->_node->session->ticket;
		}
		return $result;
	}
	
	public function getGuestUrl()
	{
		// TODO what should be returned if the content has been updated??
		
		$this->populateContentData();	
		$result = null;
		if ($this->_url != null)
		{	
			$result = $this->_url."?guest=true";
		}
		return $result;
	}
	
	public function getContent()
	{
		$this->populateContentData();
		
		$result = null;			
		if ($this->_isDirty == true)
		{
			if ($this->_newFileContent != null)
			{
				$handle = fopen($this->_newFileContent, "rb");
				$result = stream_get_contents($handle);
				fclose($handle);	
			}
			else if ($this->_newContent != null)
			{
				$result = $this->_newContent;	
			}	
		}
		else
		{
			if ($this->getUrl() != null)
			{
				$handle = fopen($this->getUrl(), "rb");
				$result = stream_get_contents($handle);
				fclose($handle);	
			}
		}
		return $result;
	}
	
	public function setContent($content)
	{
		$this->populateContentData();
		$this->_isDirty = true;
		$this->_newContent = $content;			
	}
	
	public function writeContentFromFile($fileName)
	{
		$this->populateContentData();
		$this->_isDirty = true;
		$this->_newFileContent = $fileName;		
	}
	
	public function readContentToFile($fileName)
	{
		$handle = fopen($fileName, "wb");
		fwrite($handle, $this->getContent());
		fclose($handle);	
	}
	
	public function onBeforeSave(&$statements, $where)
	{
		if ($this->_isDirty == true)
		{
			// Check mimetype has been set
			if ($this->_mimetype == null)
			{
				throw new Exception("A mime type for the content property ".$this->_property." on node ".$this->_node->__toString()." must be set");
			}
			
			// If a file has been specified then read content from there
			//$content = null;
			if ($this->_newFileContent != null)
			{
				// Upload the content to the repository
				$contentData = upload_file($this->node->session, $this->_newFileContent, $this->_mimetype, $this->_encoding);
				
				// Set the content property value
				$this->addStatement(
					$statements, 
					"update", 
					array("property" => array(
								"name" => $this->property,
								"isMultiValue" => false,
								"value" => $contentData)) + $where);	
			}
			else
			{
				// Add the writeContent statement
				$this->addStatement(
						$statements, 
						"writeContent", 
						array(
							"property" => $this->_property,
							"content" => $this->_newContent,
							"format" => array(
								"mimetype" => $this->_mimetype,
								"encoding" => $this->_encoding)) + 
							$where); 
			} 
		}
	}
	
	public function onAfterSave()
	{
		$this->_isDirty = false;
		$this->_isPopulated = false;
		$this->_mimetype = null;
		$this->__size = null;
		$this->__encoding = null;	
		$this->__url = null;
		$this->__newContent = null;
	}
	
	private function populateContentData()
	{
		//echo "isPopulated:".$this->_isPopulated."; node:".$this->_node."; property:".$this->_property."<br>";
		if ($this->_isPopulated == false && $this->_node != null && $this->_property != null && $this->_node->isNewNode == false)
		{			
			$result = $this->_node->session->contentService->read( array(
																"items" => array(
																	"nodes" => array(
																		"store" => $this->_node->store->__toArray(),
																		"uuid" => $this->_node->id)),			
																"property" => $this->_property) );
			if (isset($result->content) == true)
			{										
				if (isset($result->content->length) == true)
				{																
					$this->_size = $result->content->length;
				}
				if (isset($result->content->format->mimetype) == true)
				{																
					$this->_mimetype = $result->content->format->mimetype;
				}
				if (isset($result->content->format->encoding) == true)
				{
					$this->_encoding = $result->content->format->encoding;
				}
				if (isset($result->content->url) == true)
				{
					$this->_url = $result->content->url;
				}
			}															
			
			$this->_isPopulated = true;
		}
	}
	
	private function addStatement(&$statements, $statement, $body)
	{		
		$result = array();	
		if (array_key_exists($statement, $statements) == true)	
		{
			$result = $statements[$statement];
		}
		$result[] = $body;
		$statements[$statement] = $result;
	}
}
?>
