<?php
/*
 * Copyright 2005 - 2009  Zarafa B.V.
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License, version 3, 
 * as published by the Free Software Foundation with the following additional 
 * term according to sec. 7:
 *  
 * According to sec. 7 of the GNU Affero General Public License, version
 * 3, the terms of the AGPL are supplemented with the following terms:
 * 
 * "Zarafa" is a registered trademark of Zarafa B.V. The licensing of
 * the Program under the AGPL does not imply a trademark license.
 * Therefore any rights, title and interest in our trademarks remain
 * entirely with us.
 * 
 * However, if you propagate an unmodified version of the Program you are
 * allowed to use the term "Zarafa" to indicate that you distribute the
 * Program. Furthermore you may use our trademarks where it is necessary
 * to indicate the intended purpose of a product or service provided you
 * use it in accordance with honest practices in industrial or commercial
 * matters.  If you want to propagate modified versions of the Program
 * under the name "Zarafa" or "Zarafa Server", you may only do so if you
 * have a written permission by Zarafa B.V. (to acquire a permission
 * please contact Zarafa at trademark@zarafa.com).
 * 
 * The interactive user interface of the software displays an attribution
 * notice containing the term "Zarafa" and/or the logo of Zarafa.
 * Interactive user interfaces of unmodified and modified versions must
 * display Appropriate Legal Notices according to sec. 5 of the GNU
 * Affero General Public License, version 3, when you propagate
 * unmodified or modified versions of the Program. In accordance with
 * sec. 7 b) of the GNU Affero General Public License, version 3, these
 * Appropriate Legal Notices must retain the logo of Zarafa or display
 * the words "Initial Development by Zarafa" if the display of the logo
 * is not reasonably feasible for technical reasons. The use of the logo
 * of Zarafa in Legal Notices is allowed for unmodified and modified
 * versions of the software.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 *  
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */

?>
<?php
/*
 * Copyright (C) 2005 Alfresco, Inc.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.

 * As a special exception to the terms and conditions of version 2.0 of 
 * the GPL, you may redistribute this Program in connection with Free/Libre 
 * and Open Source Software ("FLOSS") applications as described in Alfresco's 
 * FLOSS exception.  You should have recieved a copy of the text describing 
 * the FLOSS exception, and it is also available here: 
 * http://www.alfresco.com/legal/licensing"
 */

require_once 'WebService/WebServiceFactory.php';
require_once 'BaseObject.php';

if (isset($_SESSION) == false)
{
   // Start the session
   session_start();
}   
 
class Repository extends BaseObject
{
	private $_connectionUrl;	
	private $_host;
	private $_port;

	public function __construct($connectionUrl="http://localhost:8080/alfresco/api")
	{
		$this->_connectionUrl = $connectionUrl;			
		$parts = parse_url($connectionUrl);
		$this->_host = $parts["host"];
		$this->_port = (isset($parts["port"])) ? $parts["port"] : 80;
	}
	
	public function getConnectionUrl()
	{
		return $this->_connectionUrl;
	}
	
	public function getHost()
	{
		return $this->_host;	
	}
	
	public function getPort()
	{
		return $this->_port;
	}
	
	public function authenticate($userName, $password)
	{
		// TODO need to handle exceptions!
		
		$authenticationService = WebServiceFactory::getAuthenticationService($this->_connectionUrl);
		$result = $authenticationService->startSession(array (
			"username" => $userName,
			"password" => $password
		));
		
		// Get the ticket and sessionId
		$ticket = $result->startSessionReturn->ticket;
		$sessionId = $result->startSessionReturn->sessionid;
		
		// Store the session id for later use
		if ($sessionId != null)
		{
         $sessionIds = null;
         if (isset($_SESSION["sessionIds"]) == false)
			{
            $sessionIds = array();
         }
         else
         {
            $sessionIds = $_SESSION["sessionIds"];
         }
         $sessionIds[$ticket] = $sessionId;
         $_SESSION["sessionIds"] = $sessionIds;
		}
		
		return $ticket;
	}
	
	public function createSession($ticket=null)
	{
		$session = null;
		
		if ($ticket == null)
		{
			// TODO get ticket from some well known location ie: the $_SESSION
		}	
		else
		{
			// TODO would be nice to be able to check that the ticket is still valid!
			
			// Create new session
			$session = new Session($this, $ticket);	
		}
		
		return $session;
	}
	
	/**
	 * For a given ticket, returns the realated session id, null if one can not be found.
	 */
	public static function getSessionId($ticket)
	{
      $result = null;
      if (isset($_SESSION["sessionIds"]) == true)
      {
         $result = $_SESSION["sessionIds"][$ticket];
      }
      return $result;

	}

}
 
 ?>