<?php
/*
 * Copyright 2005 - 2009  Zarafa B.V.
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License, version 3, 
 * as published by the Free Software Foundation with the following additional 
 * term according to sec. 7:
 *  
 * According to sec. 7 of the GNU Affero General Public License, version
 * 3, the terms of the AGPL are supplemented with the following terms:
 * 
 * "Zarafa" is a registered trademark of Zarafa B.V. The licensing of
 * the Program under the AGPL does not imply a trademark license.
 * Therefore any rights, title and interest in our trademarks remain
 * entirely with us.
 * 
 * However, if you propagate an unmodified version of the Program you are
 * allowed to use the term "Zarafa" to indicate that you distribute the
 * Program. Furthermore you may use our trademarks where it is necessary
 * to indicate the intended purpose of a product or service provided you
 * use it in accordance with honest practices in industrial or commercial
 * matters.  If you want to propagate modified versions of the Program
 * under the name "Zarafa" or "Zarafa Server", you may only do so if you
 * have a written permission by Zarafa B.V. (to acquire a permission
 * please contact Zarafa at trademark@zarafa.com).
 * 
 * The interactive user interface of the software displays an attribution
 * notice containing the term "Zarafa" and/or the logo of Zarafa.
 * Interactive user interfaces of unmodified and modified versions must
 * display Appropriate Legal Notices according to sec. 5 of the GNU
 * Affero General Public License, version 3, when you propagate
 * unmodified or modified versions of the Program. In accordance with
 * sec. 7 b) of the GNU Affero General Public License, version 3, these
 * Appropriate Legal Notices must retain the logo of Zarafa or display
 * the words "Initial Development by Zarafa" if the display of the logo
 * is not reasonably feasible for technical reasons. The use of the logo
 * of Zarafa in Legal Notices is allowed for unmodified and modified
 * versions of the software.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 *  
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */

?>
<?php
/**
 * Alfresco Plugin
 *
 * Connects, authenticates and loads spaces and files from Alfresco.
 * Files can also be up and downloaded.
 *
 * The PHP-Alfresco API is used (with some slightly modifications
 * to improve performance) and is located at includes/Alfresco.
 *
 * Author: Sebastian Kummer <s.kummer@zarafa.com>
 *
 */


class Pluginalfresco extends Plugin {
    /**
     * The Alfresco Repository object
     */
    var $repository;

    /**
     * The Alfresco Session object
     */
    var $session;

    /**
     * The Alfresco Ticket (received after the login)
     */
    var $ticket;

    /**
     * The Alfresco Store object
     */
    var $store;

    /**
     * The spaceid counter (dtree)
     */
    var $spaceid;


    /**
    * Constructor
    */
    function Pluginalfresco(){
        // supported file extensions (icons available) in the space browser
        $this->icons = array("acp","bmp","docx","gif","jpg","mp2","mpeg","odf","ods","png","psd","swf","txt","wpd","xml","xslx","asf","csv","exe","htm","js","mp3","mpg"," odg","odt","ppt","rtf","tiff","url","xls","xsd","zip","avi","doc","ftl","html","lnk","mp4","msg","odp","pdf","pptx","shtml","tif","wmv","xlsx","xsl");

        $this->repository = new Repository($GLOBALS['pluginconfig']['alfresco']['url'].'/'.$GLOBALS['pluginconfig']['alfresco']['api']);
        $this->session = false;
        $this->ticket = false;
        $this->ticketTime = 0;
        $this->store = false;
        $this->spaceid = false;
        define("PLUGIN_ALFRESO_ROOTSPACE", (isset($GLOBALS['pluginconfig']['alfresco']['rootspace']))?$GLOBALS['pluginconfig']['alfresco']['rootspace']:"app:company_home");

        // set default timeout to 30 minutes
        if (!$GLOBALS['pluginconfig']['alfresco']['timeout']) $GLOBALS['pluginconfig']['alfresco']['timeout'] = 30*60;
    }


    /**
    * Function initializes the Plugin and registers all hooks
    *
    * @return void
    */
    function init(){
        // on load the login data is fetched so they can be used to login at alfresco
        $this->registerHook('server.index.login.success');
        $this->registerHook('server.main.include.jsfiles');

        // event to add the 'insert Alfresco Link' to the 'new email window' menubar and the 'attachments window' menubar.
        $this->registerHook('server.dialog.general.setup.getmenubuttons');
        $this->registerHook('server.dialog.general.setup.getincludes');

        // event to add hidden fields needed to upload alfresco files in the attachments dialog
        $this->registerHook('server.dialog.attachments.setup.getbody.uploadformhtml');

        // event to attach (download) Alfresco files requested by the 'attachments' window (createmail)
        $this->registerHook('server.dialog.general.upload_attachment.before');

        // trigger the entire execute event for maillistmodule
        $this->registerHook('server.module.maillistmodule.execute.before');

        // trigger the entire execute event for AlfrescoSpaceModule (uses login information from plugin sessiondata)
        $this->registerHook('server.plugin.alfrescospacemodule.execute');
    }


    /**
    * Function is executed when a hook is triggered by the PluginManager
    *
    * @param string $eventID the id of the triggered hook
    * @param mixed $data object(s) related to the hook
    * @return void
    */
    function execute($eventID, &$data){
        switch($eventID){
            case "server.index.login.success":
                $this->login();
                break;
            case "server.main.include.jsfiles":
                $this->loadjs($data);
                break;
            case "server.dialog.general.setup.getmenubuttons":
                if($data['task'] == 'attachments'){
                    $this->attachmentAddButton($data);
                }
                else{
                    $this->modifyDialogMenu($data);
                }
                break;
            case "server.dialog.general.setup.getincludes":
                $this->dialogGetIncludes($data);
                break;
            case "server.dialog.attachments.setup.getbody.uploadformhtml":
                $this->attachmentAddHiddenFields($data);
                break;
            case "server.dialog.general.upload_attachment.before":
                $this->addAttachmentFromAlfresco($data);
                break;
            case "server.module.maillistmodule.execute.before":
                $this->moduleExecute($data['moduleObject']);
                break;
            case "server.plugin.alfrescospacemodule.execute":
                $this->moduleExecute($data['moduleObject']);
                break;
        }
    }


    /** ----------------- ALFRESCO LOGIN & SESSION STUFF ------------------------------------------------------------ */

    /**
    * Function is triggered (as well) when the login into the WA was sucessfully
    *
    * @return void
    */
    function login() {
        // only login when executed with a username in the session!
        if (! $_SESSION["username"] ) return;

        // use the logindata from the webaccess or the default login credentials from the config
        if (isset($GLOBALS['pluginconfig']['alfresco']['useUserCredentials']) && $GLOBALS['pluginconfig']['alfresco']['useUserCredentials'] == true) {
            $GLOBALS['pluginconfig']['alfresco']['defaultUser'] = $_SESSION["username"];
            if (isset($_SESSION["password"]))
                $GLOBALS['pluginconfig']['alfresco']['defaultPassword'] = $_SESSION["password"];
        }

        // try to get ticket and sessionids from the plugin session data
        $this->ticket = $this->getData("ticket");
        $_SESSION["sessionIds"] = $this->getData("sessionIds");

        // no ticket available -> authenticate at alfresco server and save session!
        if (!isset($this->ticket) || $this->ticket == "" || !is_array($_SESSION["sessionIds"]) || empty($_SESSION["sessionIds"])) {
            try {
                // try to connect directly to the server, preventing the uncatchable SOAP error..
                // as the ticket is then saved, this has to be done only once per session..
                // http://bugs.php.net/bug.php?id=34657
                $resp = $this->connect($GLOBALS['pluginconfig']['alfresco']['api']."/AuthenticationService?wsdl","","");

                // check if there is an HTTP 200 OK code
                preg_match("/http.*? ((\d+) .*)/i", $resp['header'],$m);
                if ($m[2] != "200" ) throw new Exception (dgettext('plugin_alfresco', "Unexpected HTTP code")." (". trim($m[1]) .")");

                // seems to be fine -> do the SOAP call
                $this->ticket = $this->repository->authenticate($GLOBALS['pluginconfig']['alfresco']['defaultUser'], $GLOBALS['pluginconfig']['alfresco']['defaultPassword']);

                $this->setData("ticket", $this->ticket, false);
                $this->setData("ticketTime", time());
                $this->setData("sessionIds", $_SESSION["sessionIds"]);
            }
            catch (Exception  $e) {
                if (DEBUG_PLUGINS) dump("Alfresco->login(): Exception catched: ". $e->getMessage());
                $this->setData("loginerror", dgettext('plugin_alfresco', "Authentication error") . (($e->getMessage() != "")?": " . $e->getMessage():""));
            }
        }
    }


    /**
    * Function creates a valid Alfresco-session object from the saved ticket.
    * It also initializes the store and the alfresco-session if necessary
    *
    * @throws SoapFault when no ticket is available
    * @return void
    */
    function preOperation() {
        if (!$this->ticket) {
            $this->ticket = $this->getData("ticket");
            $this->ticketTime = $this->getData("ticketTime");
            $_SESSION["sessionIds"] = $this->getData("sessionIds");
        }

        // try to relogin if the ticket is too old
        if (!$this->ticketTime || $this->ticketTime + $GLOBALS['pluginconfig']['alfresco']['timeout'] < time()) {
            if (DEBUG_PLUGINS) dump("Alfresco->preOperation(): Ticket too old, try relogin");
            $this->setData("ticket", false);
            $this->ticket = "";
            $this->login();
        }

        // throw a hard error!
        if (!$this->ticket) {
            $loginerror = $this->getData("loginerror");
            throw new SoapFault("Server", ($loginerror)?$loginerror:dgettext('plugin_alfresco', "no ticket available (reason unknown)"));
        }

        // rebuild the Alfresco Session object
        if (!$this->session) {
            $this->session = $this->repository->createSession($this->ticket);
            if (!$this->store) $this->store = new SpacesStore($this->session);

            // load last spaceid of the node in the nodecache
            $this->spaceid = $this->getData("spaceid");
            if (!isset($this->spaceid) || $this->spaceid == "") $this->spaceid = 0;

            // load and set nodecache to alfresco session
            $nodestate = $this->getData("nodecache");

            if (isset($nodestate) && is_array($nodestate)) {
                $this->session->setNodeCache($nodestate);
                foreach ($this->session->getNodeCache() as $node) {
                    // remove nodes that have no spaceid -> they will be recached at the next request
                    if (!isset($node->zarafaTree['spaceid'])) {
                        $this->session->removeNode($node);
                    }
                    // populate session and store to the node (ops.. not in the AlfresoAPI.. hack!)
                    $node->updateSession($this->session, $this->store);
                }
            }
        }
        if (!$this->store) $this->store = new SpacesStore($this->session);
    }


    /**
     * Saves the list with already processed Nodes to the plugin session for faster buildup
     *
     * @return void
     */
    function postOperation() {
        if ($this->session && $this->ticket) {
            $this->setData("nodecache", $this->session->getNodeCache(), false);
            $this->setData("ticketTime", time());
            $this->setData("spaceid", $this->spaceid);
        }
    }


    /** ----------------- GUI STUFF (BUTTONS ETC) --------------------------------------------------------------- */

    /**
    * Function is triggered when initializing the WA and adds the alfrescospacemodule (JS) so it will be always loaded
    *
    * @param mixed $data object(s) related to the hook
    * @return void
    */
    function loadjs(&$data) {
        $data["files"][] = $this->getPluginPath(). "alfrescospacemodule.js";
    }


    /**
    * Function adds the Alfresco-Link button to the menu of a dialog
    *
    * @param mixed $data object(s) related to the hook
    * @return void
    */
    function modifyDialogMenu(&$data) {
        // ignore all dialogs except "createmail"
        if ($data["task"] != "createmail") return;
        $data["buttons"][] = array(
                        'id'=>"seperator",
                        'name'=>"",
                        'title'=>"",
                        'callback'=>""
                    );
        $data["buttons"][] = array(
                        'id'=>"alfrescolink",
                        'name'=>dgettext('plugin_alfresco', "Alfresco Link"),
                        'title'=>dgettext('plugin_alfresco', "Insert link of an Alfresco file"),
                        'callback'=>"function(){
                                         webclient.openModalDialog(module, 'alfrescolink', DIALOG_URL+'task=alfrescospace&plugin=alfresco&files=true', 600,460, alfrescoLinkDialog_selectCallBack);
                                     }"
                    );
    }


    /**
    * Function gets includes for a dialog
    * Adds the alfresco.js and css files
    *
    * @param mixed $data object(s) related to the hook
    * @return void
    */
    function dialogGetIncludes(&$data) {
        // ignore all dialogs except "createmail"
        if ($data["task"] != "createmail" &&
            $data["task"] != "attachments"
            ) return;
        $data["includes"][] = $this->getPluginPath(). "plugin.alfresco.js";
        $data["includes"][] = $this->getPluginPath(). "plugin.alfresco.css";
    }


    /**
    * Function adds "add download" fieldset button in the download dialog
    *
    * @param mixed $data object(s) related to the hook
    * @return void
    */
    function attachmentAddButton($data) {
        if($data["task"] != "attachments") return;
        $data["buttons"][] = array(
            'id' => "alfresco_attachmentsdialog_addattachment",
            'name' => dgettext('plugin_alfresco', 'Add Alfresco File'),
            'title' => dgettext('plugin_alfresco', 'Add Alfresco File as attachment to this message'),
            'callback' => 'function(){webclient.openModalDialog(module, \'alfrescoattachment\', DIALOG_URL+\'task=alfrescospace&plugin=alfresco&files=true\', 600,460, alfrescoAttachmentDialog_selectCallBack);}'
        );
    }


    /**
    * Function adds two hidden fields to transmit an alfresco selected file to the WA-backend.
    *
    * @param mixed $data object(s) related to the hook
    * @return void
    */
    function attachmentAddHiddenFields($data) {
        // add hidden fields
        $data['html'] .= '<input type="hidden" id="alfrescofullpath" name="alfrescofullpath" value="" />';
        $data['html'] .= '<input type="hidden" id="alfrescolabel" name="alfrescolabel" value="" />';
    }


    /** ----------------- ACTIONS FROM THE CLIENT / ALFRESCO SPACE BROWSER --------------------------------------- */

    /**
    * Function triggers a download from the Alfresco repository and adds it to the file list (download dialog)
    *
    * @param mixed $data object(s) related to the hook
    * @return void
    */
    function addAttachmentFromAlfresco(&$data) {
        // check if request is an alfresco file
        if (!isset($_REQUEST['alfrescofullpath']) || $_REQUEST['alfrescofullpath'] == "" ||
            !isset($_REQUEST['alfrescolabel'])    || $_REQUEST['alfrescolabel'] == "") return;

        try {
            $this->preOperation();
        }
        catch (Exception $e) {
            if (DEBUG_PLUGINS) dump("Alfresco->addAttachmentFromAlfresco()->preOperation() failed");
            return;
        }

        $fullpath = $_REQUEST['alfrescofullpath'];
        $label = $_REQUEST['alfrescolabel'];
        if (!$fullpath || !$label) return;

        // Check if there are no files uploaded
        if(!isset($_SESSION["files"])) {
            $_SESSION["files"] = array();
        }

        // Check if no files are uploaded with this dialog_attachments
        if(!isset($_SESSION["files"][$_POST["dialog_attachments"]])) {
            $_SESSION["files"][$_POST["dialog_attachments"]] = array();
        }

        // Create unique tmpname
        $tmpname = tempnam(TMP_PATH, stripslashes($label));

        // Get the file from alfresco and save it to $tmpname
        $alAction = $this->downloadFile($fullpath, $tmpname);

        // stripping path details
        $tmpname = basename($tmpname);

        // Add file information to the session if download was successful
        if ($alAction['size'] != false)
            $_SESSION["files"][$_POST["dialog_attachments"]][$tmpname] = Array(  "name" => utf8_to_windows1252(stripslashes($label)),
                                                                                 "size" => $alAction['size'],
                                                                                 "type" => $alAction['type'],
                                                                                 "sourcetype" => 'alfresco',
                                                                                 "alfrescoFile" => true);
        else
            if (DEBUG_PLUGINS) dump("Alfresco->addAttachmentFromAlfresco(): failed! File could not be downloaded.");
    }


    /**
    * Function is triggered on
    *     - MailListModule->execute and
    *     - AlfrescoSpaceModule->execute
    * works with the complete module objects
    *
    * @param object $aModule related module object
    * @return void
    */
    function moduleExecute(&$aModule) {
        foreach($aModule->data as $action)
        {
            if(isset($action["attributes"]) && isset($action["attributes"]["type"])) {
                $store = $aModule->getActionStore($action);
                $parententryid = $aModule->getActionParentEntryID($action);

                switch($action["attributes"]["type"]) {
                    case "getList":
                        $result = $this->getList($aModule, $action);
                        break;
                    case "preloadData":
                        $result = $this->preLoadData($aModule, $action);
                        break;
                    case "AlfrescoArchive":
                        $result = $this->archiveFiles($aModule, $store, $parententryid, $action);
                        break;
                    case "getAttachmentsInformation":
                        $result = $this->getAttachmentsInformation($aModule, $store, $action);
                        break;
                }
            }
        }
    }


    function getAttachmentsInformation($aModule, $store, $data) {
        $atts = array();

        if ($store) {
            $message = mapi_msgstore_openentry($store, hex2bin($data['entryid']));
            if ($message) {
            	$fromname = "";
                $attachments = mapi_message_getattachmenttable($message);
                $attachTable = mapi_table_queryallrows($attachments, Array(PR_ATTACH_NUM, PR_ATTACH_CONTENT_ID));

                foreach($attachTable as $attachRow) {
                    // Open the attachment
                    $attachment = mapi_message_openattach($message, $attachRow[PR_ATTACH_NUM]);

                    // Check if the attachment is opened
                    if($attachment) {
                        // Get the props of the attachment
                        $props = mapi_attach_getprops($attachment, array(PR_ATTACH_LONG_FILENAME, PR_ATTACH_MIME_TAG, PR_DISPLAY_NAME, PR_ATTACH_METHOD, PR_ATTACH_NUM, PR_ATTACH_SIZE));
                        // Content Type
                        $contentType = "application/octet-stream";
                        // Filename
                        $filename = "ERROR";
                        // GET the name of "FROM"
                        $messageprops = mapi_getprops($message, array(PR_SENT_REPRESENTING_NAME));
			            if(isset($messageprops[PR_SENT_REPRESENTING_NAME])){
			            	$fromname = $messageprops[PR_SENT_REPRESENTING_NAME];
			            }

                        // Set filename
                        if(isset($props[PR_ATTACH_LONG_FILENAME])) {
                            $filename = $props[PR_ATTACH_LONG_FILENAME];
                        } else if(isset($props[PR_ATTACH_FILENAME])) {
                            $filename = $props[PR_ATTACH_FILENAME];
                        } else if(isset($props[PR_DISPLAY_NAME])) {
                            $filename = $props[PR_DISPLAY_NAME];
                        }

                        // Set content type
                        if(isset($props[PR_ATTACH_MIME_TAG]))
                            $contentType = $props[PR_ATTACH_MIME_TAG];
                        else
                            $contentType = $this->getFileMimeType($filename, $contentType);

                        $atts[] = array("filename" => $filename, "contenttype" => $contentType, "attachnumber" => $props[PR_ATTACH_NUM], "attachmentsize" => $props[PR_ATTACH_SIZE], "fromname" => $fromname);

                    } // end if attachment
                } // end foreach attachment in message

            }
            else {
                if (DEBUG_PLUGINS) dump("Unable to open message:".sprintf("%X"), mapi_last_hresult());
            }
        }
        else {
            if (DEBUG_PLUGINS) dump("Unable to open store:".sprintf("%X"), mapi_last_hresult());
        }

        $data["attributes"] = array("type" => "attachments");
        $data["attachments"] = $atts;

        array_push($aModule->responseData["action"], $data);
        $GLOBALS["bus"]->addData($aModule->responseData);
        return true;

    }


/**
* Function preloads the nodes from Alfresco
*
* @param object $alfrescoSpaceModule the alfresco space module
* @param array $action data transmitted from the request
* @return boolean true
*/
    function preLoadData($alfrescoSpaceModule, $action){
        $data = array();
        $c = 0;

        try {
            if (DEBUG_PLUGINS) dump("Alfresco->preLoadData(): start");
            // gets the nodes from alfresco
            $this->buildList(true, "", $c);

            // refresh the nodes to resolve possible unsolved dependencies
            $this->buildList(true, "", $c);

            // save cache
            $this->postOperation();

            $data["attributes"] = array("type" => "preLoadFinished");
            $data["nodecount"] = $c;
            if (DEBUG_PLUGINS) dump("Alfresco->preLoadData(): done! Fetched nodes: ".$c);
        }
        catch (SoapFault $e) {
            $data["attributes"] = array("type" => "preLoadError");

            // show error message after login only if allowed
            if (isset($GLOBALS['pluginconfig']['alfresco']['allowalert']) && $GLOBALS['pluginconfig']['alfresco']['allowalert'] == true)
                $data["errormsg"] = $e->getMessage();

            if (DEBUG_PLUGINS) dump("Alfresco->preLoadData(): failed! Error message: ". $e->getMessage());
        }

        array_push($alfrescoSpaceModule->responseData["action"], $data);
        $GLOBALS["bus"]->addData($alfrescoSpaceModule->responseData);
        return true;
    }


/**
* Function retrieves the list of spaces, files or executes a search on Alfresco
*
* @param object $alfrescoSpaceModule the alfresco space module
* @param array $action data transmitted from the request
* @return boolean true on success or false on failure
*/
    function getList($alfrescoSpaceModule, $action){
        $viewFiles = true;
        $searchfor = "";

        if (isset($action["viewFiles"]))
            $viewFiles = $action['viewFiles'];

        if (isset($action["searchfor"]))
            $searchfor = $action['searchfor'];

        $docsFound = 0;

        // $this->buildList throws an exception when a problem occurs
        try {
            $spacelist = $this->buildList($viewFiles, $searchfor, $docsFound);
        }
        catch (SoapFault $e) {
            $errmsg = $e->getMessage();
            if (!$errmsg) $errmsg = dgettext('plugin_alfresco', "Unknown error. Please contact your Alfresco administrator.");

            // DOMDocument is the xml parser.
            if (stripos($errmsg, "DOMDocument") !== false) $errmsg .= " - Please install the PHP-XML module on the server!";

            $brstr = '<br><img src="'. $this->getPluginPath() . 'img/empty.gif" height=0 width=39>';
            $names = array();
            $names['spaceid'] = 1;
            $names['parentid'] = 0;
            $names['label'] =  " ". dgettext('plugin_alfresco', "Error fetching data from Alfresco").": ". $brstr . wordwrap($errmsg, 85, $brstr) ;
            $names['title'] = "";
            $names['fullpath'] = "";
            $names['size'] = null;
            $names['isdocument'] = true;
            $names['icon'] = "../icon_close";

            $spacelist = array($names);
        }

        $data = array("imguri" => $this->getPluginPath(), "spaces" =>  $spacelist, "found" => $docsFound );

        // expand all spaces if searching
        if ($searchfor != "") $data["autoexpand"] = "open";

        // save cache
        $this->postOperation();

        $data["attributes"] = array("type" => "listspaces");
        array_push($alfrescoSpaceModule->responseData["action"], $data);
        $GLOBALS["bus"]->addData($alfrescoSpaceModule->responseData);
        return true;
    }


    function archiveFiles(&$maillist, $store, $parententryid, $action) {
        $attcount = 0;
        $filesStored = '';
        $noFilesStored = '';
        $msg = '';
        try {
            $this->preOperation();
        }
        catch (Exception $e) {
            if (DEBUG_PLUGINS) dump("Alfresco->archiveFiles()->preOperation() failed");
            return;
        }

        $message = mapi_msgstore_openentry($store, hex2bin($action['entryid']));

        if ($message) {
            //if there's only one file to archive, the fields are directly under $action['attachments']
            //in such a case the fields are moved to $action['attachments'][0], so we can loop them in
            //the same way as multiple files
            if (isset($action['attachments']['filemetaname'])) {
                $action['attachments'][0] = $action['attachments'];
                unset($action['attachments']['filemetaname'], $action['attachments']['filemetatitle'], $action['attachments']['filemetadescr'], $action['attachments']['filemetaauthor'], $action['attachments']['filenumber']);
            }

            foreach ($action['attachments'] as $file) {
                $attachment = mapi_message_openattach($message, $file['filenumber']);
                // Check if the attachment is opened
                if($attachment) {
                    // Get the props of the attachment
                    $props = mapi_attach_getprops($attachment, array(PR_ATTACH_MIME_TAG));
                    $file['contenttype'] = "application/octet-stream";

                    // Set content type
                    if(isset($props[PR_ATTACH_MIME_TAG]))
                        $file['contenttype'] = $props[PR_ATTACH_MIME_TAG];
                    else
                        $file['contenttype'] = $this->getFileMimeType($file['filemetaname'], $contentType);

                    // Open a stream to get the attachment data
                    $stream = mapi_openpropertytostream($attachment, PR_ATTACH_DATA_BIN);

                    if ($stream !== false) {
                        if (($msg = $this->uploadFile($action['node'], $file, $stream)) === true) {
                            $attcount++;
                            $filesStored .= $file['filemetaname'] . " - " .dgettext('plugin_alfresco', "OK") . "\n";
                        }
                        else{
                        	$filesStored .= $file['filemetaname'] . " - " . dgettext('plugin_alfresco', "Error") . " ($msg)\n";
                        }
                    }
                }
                else {
                    if (DEBUG_PLUGINS) dump("Unable to open attachment {$file['filenumber']} {$action['attachments']['filemetaname']}:".sprintf("%X"), mapi_last_hresult());
                }
            }

        }
        else {
                if (DEBUG_PLUGINS) dump("Unable to open message:".sprintf("%X"), mapi_last_hresult());
        }
        $response = array();
        $response["archivedAttachments"] = $attcount;
        $response["stored"] = $filesStored;
        $response["attributes"] = array("type" => "AlfrescoArchiveResponse");
        array_push($maillist->responseData["action"], $response);
        $GLOBALS["bus"]->addData($maillist->responseData);
    }

    /** ----------------- RAW UPLOADING AND DOWNLOADING FUNCTIONS --------------------------------------------------- */

    /**
     * Downloads a file from Alfresco and saves it to the passed $destinationFile
     *
     * @param string $alfrescoPath  path of the file location
     * @param string $destinationFile path where the file should be saved to
     * @return array with meta information about the file
     */
    function downloadFile($alfrescoPath, $destinationFile) {
        $dl = $GLOBALS['pluginconfig']['alfresco']['apiDownload'] . ((substr($GLOBALS['pluginconfig']['alfresco']['apiLink'],-1, 1) != "/")?"/":"");

        $rheader = $this->connect($dl . $alfrescoPath, "", "", array(), "ticket=". $this->ticket, $destinationFile);

        // parse response headers for size and file-type
        $size = false;
        $type = "";
        if (isset($rheader["header"])) {
            $rheaders = explode("\r\n", $rheader["header"]);
            foreach($rheaders as $h) {
                if (stripos($h, "Content-length") === 0) list(,$size) = explode(":", $h,2);
                if (stripos($h, "Content-type") === 0) list(,$type) = explode(":", $h,2);

            }
        }
        if ($size == 0) $size = filesize($destinationFile);
        if (($pos=strpos($type,";")) > 0) $type = substr($type, 0, $pos);

        return array("size"=>trim($size), "type"=>trim($type));
    }



    /**
     * Uploads a file to Alfresco
     *
     * @param string $destId the destination location of the new file
     * @param array $fileMetaData contains the meta information of the file
     * @param mapistream $mapiStream the mapistream containing the raw (binary) data of the file
     *
     * @return boolean
     */
    function uploadFile($destId, $fileMetaData, $mapiStream){
        $node = $this->session->getNode($this->store, $destId);
        $upload = $node->createChild('cm_content', 'cm_contains', $fileMetaData['filemetaname']);
        $upload->cm_name = $fileMetaData['filemetaname'];
        $contentData = new ContentData($upload,"{http://www.alfresco.org/model/content/1.0}content");
        $contentData->mimetype = $fileMetaData['contenttype'];
        $upload->cm_description = $fileMetaData['filemetadescr'];
        $upload->cm_title = $fileMetaData['filemetatitle'];
        $upload->cm_author = $fileMetaData['filemetaauthor'];
        // that's a bit of a hack but used to do pass a mapiStream without changing the Alfresco-PHP-API completely.
        // Instead of the raw filename put in an array containing the mapiStream which is going to be read
        // by a modified upload_file() function (at the end of this file, outside the class)
        $contentData->writeContentFromFile((array(&$mapiStream, $fileMetaData['filemetaname'])));
        $upload->cm_content = $contentData;

        try {
            $this->session->save();
            return true;
        }
        catch (Exception $e) {

            $message_error = $e->detail->RepositoryFault->message;

            if(stripos($message_error, 'Duplicate child') !== false){
                $msg_error = dgettext('plugin_alfresco', 'File already exists');
            }
            else if(stripos($message_error,'AccessDenied') !== false){
                $msg_error = dgettext('plugin_alfresco', 'You do not have permissions to save in this folder');
            }
            else{
               $msg_error = $message_error;
            }

            // in case of an upload error, clear the session before uploading next attachment
            $this->session = false;
            $this->preOperation();

            if (DEBUG_PLUGINS) dump("Alfresco->uploadFile(): error saving file '". $fileMetaData['filemetaname'] ."' to Alfresco!: ". $msg_error);
            return $msg_error;
        }

        return true;

    }

    /**
     * Connects to Alfresco for raw operations
     *
     * @param string $action the action to be executed on the remote server
     * @param string $content_type the content type of the request
     * @param string $data data to be sent
     * @param array $headers additional headers to be sent in the request
     * @param string $query an additional query string
     * @param string $destinationFile if the parameter is set, the raw data returned from the request will be saved in the file
     * @return array with header and content or status information
     */
    function connect($action, $content_type, $data, $headers = array(), $query = false, $destinationFile = false) {
        $urlParts = parse_url($GLOBALS['pluginconfig']['alfresco']['url']);

        if(isset($urlParts['port']))
            $port = $urlParts['port'];
        else {
            switch( $urlParts['scheme'] ) {
                case 'http':
                    $port = 80; // default for http
                    break;
                case 'https':
                    $port = 443; // default for https
                    break;
            }
        }

        // add action
        if (substr($urlParts['path'],-1, 1) != "/") $urlParts['path'] .= "/";
        $urlParts['path'] .= $action ."/?user=". $GLOBALS["mapisession"]->getUserName();
        if ($query) $urlParts['path'] .= "&" . $query;

        // BUILD THE HEADER ----------------------------------------------------
        $header = (($data)?"POST":"GET") . " ". $urlParts['path'] . " HTTP/1.0\r\n";
        $header .= "Host: {$urlParts['host']}\r\n";
        if ($content_type)
            $header .= "Content-type: $content_type\r\n";

        if ($data)
            $header .= "Content-length: " . strlen($data) . "\r\n";    // no $data manipulation after that point
        foreach($headers as $h) $header .= $h . "\r\n";
        $header .= "\r\n";

        if ($urlParts['scheme'] == "https") $urlParts['scheme'] = "ssl://";
        else $urlParts['scheme'] = "";

        $ip = $urlParts['host'];

        // basic regexp check if hostname or ip given
        if (!preg_match("/(?:\d{1,3}\.){3}\d{1,3}/",$urlParts['host'])) {
            $ip = gethostbyname($urlParts['host']);

            // on failure, the hostname is returned
            if ($ip == $urlParts['host'])
                throw new Exception("Can't resolve hostname: ".$urlParts['host']);
        }

        // fsockopen throws an error without error msg if the host can't be resolved
        $fp = @fsockopen($urlParts['scheme'] . $ip, $port, $errno, $errstr, 3);
        if (!$fp) {
            throw new Exception("Can't connect to ".$urlParts['host'] .":$port". (strlen($errstr)>0?": $errstr": ""));
        }

        // send !
        fputs($fp, $header . $data);

        // PROCESS ANSWER ----------------------------------------------------

        // open the destination file
        if ($destinationFile) $fdata = fopen($destinationFile, "wb");
        else $response = "";

        $rheader = false;
        $atmp = "";
        // loop through received data from alfresco
        while(!feof($fp)) {
            $a = fgets($fp, 4096);
            if (! $rheader) {
                $atmp .= $a;
                $splitted = explode("\r\n\r\n", $atmp,2);
                // headers complete?
                if (count($splitted) > 1) {
                    $rheader = $splitted[0];
                    $data = $splitted[1];
                }
            }
            else
                $data = $a;

            if ($data) {
                // write data
                if ($destinationFile) fwrite($fdata, $data);
                else $response .= $data;
            }
        }
        fclose($fp);
        if ($destinationFile) fclose($fdata);

        $ret = array("header"=>$rheader);
        if (!$destinationFile) $ret["response"] = $response;
        return $ret;
    }



    /** ----------------- TREE BUILDING FUNCTIONS --------------------------------------------------- */

    /**
     * Executes a search (or a simple list) on the Alfresco-Store.
     * Data is retrieved on the fly, but processed using existing Nodes from the cache.
     *
     * @param boolean $files indicates if files should be displayed in the returned tree or not
     * @param string $searchFor searchstring when searching for files
     * @param string $startNode fetches all nodes below the given node (not implemented)
     * @param integer $startId spaceid of the $startNode (parent identification) (not implemented)
     * @param integer $idCounter last value of the space-id so there are no conflicts with an already build tree (not used/implemented)
     * @param integer $levels indicates how many levels (deep) should be fetched in this request (not implemented-> by default all)
     * @return array $tree dtree compatible tree
     */
    function buildList($files = true, $searchfor = "", &$docsFound, $startNode = "", $startId = 0, $idCounter = 0, $levels = 2) {
        $tree = array();

        // build environment
        $this->preOperation();

        // root folder
        $rootnode = $this->store->getCompanyHome();

        // is the rootfolder already cached?
        if (!isset($rootnode->zarafaTree['spaceid']))
            $this->session->setZarafaTree($rootnode, array('spaceid' => $this->spaceid));

        if ($searchfor != "") {
            // perform search on alfresco
            $nodes = $this->session->query($this->store, ('@cm\\:name:"*'.$searchfor.'*"'));
        }
        else {
            // get the complete tree
            $nodes = $this->session->query($this->store, "PATH:\"/". PLUGIN_ALFRESO_ROOTSPACE. "//*\"");
        }
        $inTree = array();

        // build tree
        foreach ($nodes as $node) {
            //only handle folders and content
            $nodetype = $node->getType();
            if ($nodetype != "{http://www.alfresco.org/model/content/1.0}content" && $nodetype != "{http://www.alfresco.org/model/content/1.0}folder") continue;

            // build the tree for the parent nodes of the the found nodes
            if ($searchfor != "") {
                // don't show folders matching the searchstring
                if (! $this->isDocument($node)) continue;
                $this->treeForNode($node->getPrimaryParent(), $tree, $inTree);
            }

            // node was already processed -> get it from cache
            if (isset($node->zarafaTree['spaceid'])) {
                // ignore documents if displaying the spacebrowser
                if (!$files && $node->zarafaTree['isdocument'] == true)    continue;
                // add node to tree
                $tree[] = $node->zarafaTree;
                $inTree[] = $node->id;
            }
            // new node -> check if parent is known. If so, add to the cache and spacebrowser
            else {
                // ignore documents if displaying the spacebrowser
                if (!$files && $this->isDocument($node)) continue;

                //if the user is not allowed to access the parent node, an exeption is thrown
                //catch it and attach the node to the root
                try {
                    $pid = $this->isParentKnown($node);
                }
                catch (SoapFault $e) {
                    $pid = 0;
                }

                if (isset($pid))  $this->mapNode($node, $pid, $tree, $inTree);
            }
            // count the found documents/spaces
            if (isset($docsFound)) {
                // ignore spaces when counting files
                if ($files && !$this->isDocument($node)) continue;
                $docsFound++;
            }
        }

        return $tree;
    }


    /**
     * Checks if the parent a known node of the cache
     *
     * @param object $node an Alfresco node object
     * @return integer spaceid of the parent node
     */
    function isParentKnown(&$node) {
        return (isset($node->getPrimaryParent()->zarafaTree['spaceid']))?$node->getPrimaryParent()->zarafaTree['spaceid']:null;
    }


    /**
     * is this node a document (else a folder) ?
     *
     * @param object $node an Alfresco node object
     * @return boolean
     */
    function isDocument(&$node) {
        return (substr($node->type, -7) == "content");
    }


    /**
     * build a zarafaTree array and add it to the session/nodecache
     *
     * @param object $node an Alfresco node object
     * @param integer $parentid the spaceid of the parent node
     * @param array $tree the tree array
     * @param array $inTree identifies which nodes are already present in the tree
     * @return void
     */
    function mapNode($node, $parentid, &$tree, &$inTree) {
        $this->spaceid++;

        $names = array();
        $names['spaceid'] = $this->spaceid;
        $names['parentid'] = $parentid;
        $names['label'] = $node->cm_name;
        $names['title'] = $node->cm_description;
        $names['fullpath'] = "";
        $names['size'] = 0;
        $names['isdocument'] = false;
        $names['icon'] = "";

        if ($this->isDocument($node)) {
            // older Alfresco versions return content url and size in the main query
            if (isset($node->cm_content) && isset($node->cm_content->url)) {
                $startticket = strpos($node->cm_content->url, "?ticket");
                $startworkspace = strpos($node->cm_content->url, "workspace/");
                $rawurl = ($startticket && $startworkspace) ? substr($node->cm_content->url, $startworkspace, ($startticket - $startworkspace)) : $node->cm_content->url;
                $names['size'] = $node->cm_content->size;
            }
            // need to request some properties explicitly for Alfresco 3.34a
            else {
                $np = $node->getProperties();
                $rawurl = $np['{http://www.alfresco.org/model/system/1.0}store-protocol'] . "/" .
                          $np['{http://www.alfresco.org/model/system/1.0}store-identifier'] . "/".
                          $np['{http://www.alfresco.org/model/system/1.0}node-uuid'] . "/" .$node->cm_name;

                $cd = new ContentData($node, "{http://www.alfresco.org/model/content/1.0}content");
                $names['size'] = $cd->getSize();
            }
            $names['fullpath'] = $rawurl;
            $names['isdocument'] = true;
            $names['icon'] = "";
            $icon = substr($rawurl, strrpos($rawurl, '.')+1);
            if (in_array($icon, $this->icons))
                $names['icon'] = $icon;

        }
        else {
            $names['fullpath'] = $node->id;
        }

        $tree[] = $names;
        $inTree[] = $node->id;

        //add to mapping
        $this->session->setZarafaTree($node, $names);
    }


    /**
     * Builds recursively a tree for a found node (used when searching)
     *
     * @param object $node an Alfresco node object
     * @param array $tree the tree array
     * @param array $inTree identifies which nodes are already present in the tree
     * @return void
     */
    function treeForNode($node, &$tree, &$inTree) {
        if (! $node instanceof Node) return;

        if (isset($node->zarafaTree['parentid']) && $node->zarafaTree['parentid'] != 0) {
            $this->treeForNode($node->getPrimaryParent(), $tree, $inTree);
        }
        if (!in_array($node->id, $inTree)) {
            $tree[] = $node->zarafaTree;
            $inTree[] = $node->id;
        }
        return;
    }


    /** ----------------- UTILS --------------------------------------------------------------------- */

    /**
     * returns the contentType for a filename
     *
     * @param string $filename the filename
     * @param string $defaultcontentType the default content type
     * @return string content-type
     */
    function getFileMimeType($filename, $defaultcontentType) {
        $contentType = $defaultcontentType;

        // Parse the extension of the filename to get the content type
        if(strrpos($filename, ".") !== false) {
            $extension = strtolower(substr($filename, strrpos($filename, ".")));
            $contentType = "application/octet-stream";
            if (is_readable("mimetypes.dat")){
                $fh = fopen("mimetypes.dat","r");
                $ext_found = false;
                while (!feof($fh) && !$ext_found){
                    $line = fgets($fh);
                    preg_match("/(\.[a-z0-9]+)[ \t]+([^ \t\n\r]*)/i", $line, $result);
                    if ($extension == $result[1]){
                        $ext_found = true;
                        $contentType = $result[2];
                    }
                }
                fclose($fh);
            }
        }
        return $contentType;
    }
}


/**
 *
 * REPLACES THE ORIGINAL upload_file FUNCTION  FROM include/Alfresco/Service/Functions.php
 *
 * Partially copyrighted as the Alfresco PHP API
 *
 * Uploads a file into content store and returns the content data string which
 * can be used to populate a content property.
 *
 * @param $session the session
 * @param $mapiStreamAndFilename array containing the MapiStream and the filename
 * @return String the content data that can be used to update the content property
 */
function upload_file($session, $mapiStreamAndFilename, $mimetype=null, $encoding=null)
{
    // unpack the hidden mapiStream and fileName for further processing
    if (is_array($mapiStreamAndFilename)) {
        $mapiStream = $mapiStreamAndFilename[0];
        $fileName = $mapiStreamAndFilename[1];
    }

    $result = null;

    // File length
    $stat = mapi_stream_stat($mapiStream);
    $fileSize = $stat["cb"];

    // Get the address and the
    $host = $session->repository->host;
    $port = $session->repository->port;

    // Get the IP address for the target host
    $address = gethostbyname($host);

    // Create a TCP/IP socket
    $socket = socket_create(AF_INET, SOCK_STREAM, SOL_TCP);
    if ($socket === false) {
        throw new Exception ("socket_create() failed.\nReason: " . socket_strerror(socket_last_error()));
    }

    // Connect the socket to the repository
    $result = socket_connect($socket, $address, $port);
    if ($result === false)     {
        throw new Exception("socket_connect() failed.\nReason: ($result) " . socket_strerror(socket_last_error($socket)));
    }

    // Write the request header onto the socket
    $url = "/alfresco/upload/".urlencode($fileName)."?ticket=".$session->ticket;
    if ($mimetype != null)
    {
        // Add mimetype if specified
        $url .= "&mimetype=".$mimetype;
    }
    if ($encoding != null)
    {
        // Add encoding if specified
        $url .= "&encoding=".$encoding;
    }

    $in = "PUT ".$url." HTTP/1.1\r\n".
                  "Content-Length: ".$fileSize."\r\n".
                  "Host: ".$address.":".$port."\r\n".
                  "Connection: Keep-Alive\r\n".
                  "\r\n";
    socket_write($socket, $in, strlen($in));

    // Write the content found in the file onto the socket
    for($i = 0; $i < $stat["cb"]; $i += BLOCK_SIZE) {
        $content = mapi_stream_read($mapiStream, BLOCK_SIZE);
        socket_write($socket, $content, strlen($content));
    }

    // Read the response
    $recv = socket_read ($socket, 2048, PHP_BINARY_READ);
    $index = strpos($recv, "contentUrl");
    if ($index !== false) {
        $result = substr($recv, $index);
    }

    // Close the socket
    socket_close($socket);

    return $result;
}

?>